@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.oidc.impl;
 
+import static com.openexchange.java.Autoboxing.I;
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.HashMap;
@@ -34,7 +35,8 @@ import org.json.JSONWriter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import com.openexchange.ajax.LoginServlet;
-import com.openexchange.ajax.SessionUtility;import com.openexchange.ajax.login.LoginRequestContext;
+import com.openexchange.ajax.SessionUtility;
+import com.openexchange.ajax.login.LoginRequestContext;
 import com.openexchange.ajax.login.LoginRequestHandler;
 import com.openexchange.ajax.writer.ResponseWriter;
 import com.openexchange.authentication.Authenticated;
@@ -54,6 +56,7 @@ import com.openexchange.oidc.OIDCBackendConfig;
 import com.openexchange.oidc.OIDCBackendConfig.AutologinMode;
 import com.openexchange.oidc.OIDCBackendProperty;
 import com.openexchange.oidc.osgi.Services;
+import com.openexchange.oidc.state.StateManagement;
 import com.openexchange.oidc.tools.OIDCTools;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.session.Session;
@@ -65,7 +68,6 @@ import com.openexchange.tools.servlet.AjaxExceptionCodes;
 import com.openexchange.tools.session.ServerSessionAdapter;
 import com.openexchange.user.User;
 import com.openexchange.user.UserService;
-import static com.openexchange.java.Autoboxing.I;
 
 /**
  * {@link OIDCLoginRequestHandler} Performs a login with a valid {@link Reservation} and
@@ -85,10 +87,11 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
      * Initializes a new {@link OIDCLoginRequestHandler}.
      *
      * @param backend The back-end to use
+     * @param stateManagement A reference to the state management service
      */
-    public OIDCLoginRequestHandler(OIDCBackend backend) {
+    public OIDCLoginRequestHandler(OIDCBackend backend, StateManagement stateManagement) {
         super();
-        this.handler = new OIDCHandler(backend);
+        this.handler = new OIDCHandler(backend, stateManagement);
     }
 
     /**
@@ -153,6 +156,7 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
     public static class OIDCHandler {
 
         private final OIDCBackend backend;
+        private final StateManagement stateManagement;
 
         @SuppressWarnings("hiding")
         static final Logger LOG = LoggerFactory.getLogger(OIDCLoginRequestHandler.OIDCHandler.class);
@@ -161,8 +165,9 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
          * Initializes a new {@link OIDCHandler}.
          * @param backend The {@link OIDCBackend} to use
          */
-        public OIDCHandler(OIDCBackend backend) {
+        public OIDCHandler(OIDCBackend backend, StateManagement stateManagement) {
             this.backend = backend;
+            this.stateManagement = stateManagement;
         }
 
         /**
@@ -227,6 +232,12 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
                             return false;
                         }
                         backend.updateSession(session, reservation.getState());
+                        /*
+                         * keep remembering oidc -> ox session id mapping in state
+                         */
+                        if (backend.getBackendConfig().isBackchannelLogoutEnabled() || backend.getBackendConfig().isFrontchannelLogoutEnabled()) {
+                            OIDCTools.rememberSessionIds(stateManagement, session, reservation.getState().get(OIDCTools.IDTOKEN));
+                        }
                         // the getRedirectLocationForSession does also the validation check of the session
                         if (respondWithJson) {
                             this.writeSessionDataAsJson(session, response);
@@ -342,6 +353,13 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
             LoginResult result = loginUser(request, context, user, reservation.getState());
             Session session = performSessionAdditions(result, request, response, idToken);
 
+            /*
+             * extract and remember oidc session to ox session mapping as needed
+             */
+            if (backend.getBackendConfig().isBackchannelLogoutEnabled() || backend.getBackendConfig().isFrontchannelLogoutEnabled()) {
+                OIDCTools.rememberSessionIds(stateManagement, session, idToken);
+            }
+
             sendRedirect(session, request, response, respondWithJson);
         }
 
@@ -426,6 +444,14 @@ public class OIDCLoginRequestHandler implements LoginRequestHandler {
          */
         private LoginResult loginUser(HttpServletRequest request, final Context context, final User user, final Map<String, String> state) throws OXException {
             LOG.trace("loginUser(HttpServletRequest request: {}, final Context context: {}, final User user: {}, final Map<String, String> state.size: {})", request.getRequestURI(), I(context.getContextId()), I(user.getId()), I(state.size()));
+            {
+                String contextInfo = state.get(OIDCTools.CONTEXT_LOGIN_INFO);
+                String userName = state.get(OIDCTools.USER_LOGIN_INFO);
+                if (Strings.isNotEmpty(userName) && Strings.isNotEmpty(contextInfo)) {
+                    request.setAttribute(OIDCTools.CONTEXT_LOGIN_INFO, contextInfo);
+                    request.setAttribute(OIDCTools.USER_LOGIN_INFO, userName);
+                }
+            }
             final LoginRequest loginRequest = backend.getLoginRequest(request, user.getId(), context.getContextId(), backend.getLoginConfiguration());
 
             final OIDCBackend colosureBackend = backend;
