@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -20,9 +20,12 @@
  */
 package com.openexchange.oidc.impl;
 
+import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.oidc.tools.OIDCTools.tokeRequestInfo;
 import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Map;
 import java.util.concurrent.TimeUnit;
@@ -33,6 +36,7 @@ import org.apache.http.client.utils.URIBuilder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import com.nimbusds.jwt.JWT;
+import com.nimbusds.jwt.JWTParser;
 import com.nimbusds.oauth2.sdk.AuthorizationCode;
 import com.nimbusds.oauth2.sdk.AuthorizationCodeGrant;
 import com.nimbusds.oauth2.sdk.AuthorizationGrant;
@@ -57,6 +61,8 @@ import com.nimbusds.openid.connect.sdk.Nonce;
 import com.nimbusds.openid.connect.sdk.OIDCTokenResponse;
 import com.nimbusds.openid.connect.sdk.OIDCTokenResponseParser;
 import com.nimbusds.openid.connect.sdk.claims.IDTokenClaimsSet;
+import com.nimbusds.openid.connect.sdk.claims.LogoutTokenClaimsSet;
+import com.openexchange.ajax.AJAXServlet;
 import com.openexchange.ajax.LoginServlet;
 import com.openexchange.ajax.SessionUtility;
 import com.openexchange.ajax.fields.LoginFields;
@@ -84,11 +90,13 @@ import com.openexchange.rest.client.httpclient.HttpClientService;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 import com.openexchange.session.oauth.RefreshResult;
+import com.openexchange.session.oauth.RefreshResult.SuccessReason;
 import com.openexchange.session.oauth.SessionOAuthTokenService;
 import com.openexchange.session.oauth.TokenRefreshConfig;
 import com.openexchange.session.reservation.SessionReservationService;
 import com.openexchange.sessiond.ExpirationReason;
 import com.openexchange.sessiond.SessionExceptionCodes;
+import com.openexchange.sessiond.SessionFilter;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.servlet.http.Tools;
 
@@ -182,10 +190,17 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
                     if (null != sessionOAuthTokenService) {
                         OIDCBackendConfig config = backend.getBackendConfig();
                         OIDCTokenRefresher refresher = new OIDCTokenRefresher(backend, session);
-                        TokenRefreshConfig refreshConfig = OIDCTools.getTokenRefreshConfig(config);
+                        TokenRefreshConfig refreshConfig = TokenRefreshConfig.newBuilder(OIDCTools.getTokenRefreshConfig(config))
+                            .setForcedRefresh(true).build(); // force token refresh during auto-login 
                         try {
                             RefreshResult result = sessionOAuthTokenService.checkOrRefreshTokens(session, refresher, refreshConfig);
                             if (result.isSuccess()) {
+                                /*
+                                 * if successfully refreshed, keep remembering oidc -> ox session id mapping in state
+                                 */
+                                if (SuccessReason.REFRESHED.equals(result.getSuccessReason()) && (config.isFrontchannelLogoutEnabled() || config.isBackchannelLogoutEnabled())) {
+                                    OIDCTools.rememberSessionIds(stateManagement, session, (String) session.getParameter(OIDCTools.IDTOKEN));
+                                }
                                 return this.getRedirectLocationForSession(request, session);
                             }
                             LOG.debug("Error checking and refreshing tokens for session {}", session.getSessionID());
@@ -311,7 +326,7 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
     }
 
     private OIDCTokenResponse getTokenResponse(TokenRequest tokenReq) throws OXException {
-        LOG.trace("OIDCTokenResponse getTokenResponse(TokenRequest tokenReq {})", tokenReq);
+        LOG.trace("OIDCTokenResponse getTokenResponse(TokenRequest tokenReq {})", tokeRequestInfo(tokenReq));
         HTTPRequest httpRequest = this.backend.getHttpRequest(tokenReq.toHTTPRequest());
         HTTPResponse httpResponse = null;
         try {
@@ -469,38 +484,41 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
         //logout user
         Session session = this.getSessionFromId(sessionId);
         if (null == session) {
-            OXException oxe = SessionExceptionCodes.SESSION_EXPIRED.create(sessionId);
-            oxe.setProperty(SessionExceptionCodes.OXEXCEPTION_PROPERTY_SESSION_EXPIRATION_REASON, ExpirationReason.NO_SUCH_SESSION.getIdentifier());
-            throw oxe;
+            LOG.info("Session \"{}\" from logout request with state \"{}\" no longer found, redirecting without session info.", sessionId, logoutRequestInfo.getState());
+            return getRedirectForLogoutFromOXServer(request, response, logoutRequestInfo.getDomainName(), "");
         }
-        return this.getRedirectForLogoutFromOXServer(session, request, response, logoutRequestInfo);
+        return getRedirectForLogoutFromOXServer(session, request, response, logoutRequestInfo);
     }
 
     private String getRedirectForLogoutFromOXServer(Session session, HttpServletRequest request, HttpServletResponse response, LogoutRequestInfo logoutRequestInfo) throws OXException {
-        LOG.trace("getRedirectForLogoutFromOXServer(Session session: {}, HttpServletRequest request: {}, HttpServletResponse response, LogoutRequestInfo logoutRequestInfo.domainname: {})", session.getSessionID(), request.getRequestURI(), logoutRequestInfo != null ? logoutRequestInfo.getDomainName() : "null");
-        String domainName = OIDCTools.getDomainName(request, services.getOptionalService(HostnameService.class));
-        String sessionId = session.getSessionID();
+        LOG.trace("getRedirectForLogoutFromOXServer(Session session: {}, HttpServletRequest request: {}, HttpServletResponse response, LogoutRequestInfo logoutRequestInfo.domainname: {})", 
+            null == session ? "null" : session.getSessionID(), request.getRequestURI(), logoutRequestInfo != null ? logoutRequestInfo.getDomainName() : "null");
 
-        if (logoutRequestInfo != null) {
-            domainName = logoutRequestInfo.getDomainName();
-            sessionId = logoutRequestInfo.getSessionId();
+        if (null != logoutRequestInfo) {
+            return getRedirectForLogoutFromOXServer(request, response, logoutRequestInfo.getDomainName(), logoutRequestInfo.getSessionId());
         }
-        String redirectionURI = "";
+        return getRedirectForLogoutFromOXServer(request, response, OIDCTools.getDomainName(request, services.getOptionalService(HostnameService.class)), null != session ? session.getSessionID() : null);
+    }
+
+    private String getRedirectForLogoutFromOXServer(HttpServletRequest request, HttpServletResponse response, String domainName, String sessionId) throws OXException {
+        LOG.trace("getRedirectForLogoutFromOXServer(HttpServletRequest request: {}, HttpServletResponse response, domainname: {}, sessionId: {})", request.getRequestURI(), domainName, sessionId);
+
+        Tools.disableCaching(response);
+        String redirectionURI;
         try {
-            URIBuilder redirectLocation = new URIBuilder()
+            redirectionURI = new URIBuilder()
                 .setScheme(OIDCTools.getRedirectScheme(request))
                 .setHost(domainName)
                 .setPath(OIDCTools.getRedirectPathPrefix() + "login")
-                .setParameter(LoginServlet.PARAMETER_SESSION, sessionId)
-                .setParameter(LoginServlet.PARAMETER_ACTION, OIDCTools.OIDC_LOGOUT + OIDCTools.getPathString(this.backend.getPath()));
-            Tools.disableCaching(response);
-            redirectionURI = redirectLocation.build().toString();
-            LOG.trace("Logout URI for logout from OXServer: {}", redirectionURI);
-            return redirectionURI;
+                .setParameter(AJAXServlet.PARAMETER_SESSION, sessionId)
+                .setParameter(AJAXServlet.PARAMETER_ACTION, OIDCTools.OIDC_LOGOUT + OIDCTools.getPathString(backend.getPath()))
+            .build().toString();
         } catch (URISyntaxException e) {
-            this.logoutInCaseOfError(sessionId, request, response);
+            logoutInCaseOfError(sessionId, request, response);
             throw OIDCExceptionCode.UNABLE_TO_PARSE_URI.create(e, "automated construction of logout request URI");
         }
+        LOG.trace("Logout URI for logout from OXServer: {}", redirectionURI);
+        return redirectionURI;
     }
 
     @Override
@@ -532,6 +550,103 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
         }
     }
 
+    @Override
+    public void backchannelLogout(HttpServletRequest request, HttpServletResponse response) throws OXException {
+        LOG.trace("backchannelLogout(HttpServletRequest request: {}, HttpServletResponse response)", request.getRequestURI());
+        if (false == backend.getBackendConfig().isBackchannelLogoutEnabled()) {
+            throw OIDCExceptionCode.INVALID_LOGOUT_REQUEST.create("Backchannel logout disabled by configuration");
+        }
+        /*
+         * parse oidc session from logout token
+         */
+        String logoutTokenParameter = request.getParameter("logout_token");
+        if (Strings.isEmpty(logoutTokenParameter)) {
+            throw OIDCExceptionCode.INVALID_LOGOUT_REQUEST.create("No logout_token request parameter found");
+        }
+        JWT jwt;
+        try {
+            jwt = JWTParser.parse(logoutTokenParameter);
+        } catch (java.text.ParseException e) {
+            LOG.warn("Error parsing {}: {}", logoutTokenParameter, e.getMessage(), e);
+            throw OIDCExceptionCode.LOGOUTTOKEN_VALIDATON_FAILED.create(e, e.getMessage());
+        }
+        LogoutTokenClaimsSet logoutToken;
+        try {
+            logoutToken = backend.validateLogoutToken(jwt);
+        } catch (OXException e) {
+            LOG.warn("Error validating {}: {}", logoutTokenParameter, e.getMessage(), e);
+            throw OIDCExceptionCode.LOGOUTTOKEN_VALIDATON_FAILED.create(e, e.getMessage());
+        }
+        String sid = null != logoutToken.getSessionID() ? logoutToken.getSessionID().getValue() : null;
+        if (Strings.isEmpty(sid)) {
+            LOG.warn("No sid in logout token {}, unable to process backchannel logout", logoutTokenParameter);
+            throw OIDCExceptionCode.INVALID_LOGOUT_REQUEST.create("No sid in logout_token");
+        }
+        LOG.trace("Successfully parsed sid \"{}\" from logout token", sid);
+        /*
+         * lookup matching ox session & logout
+         */
+        String sessionId = stateManagement.getAndRemoveOXSessionId(sid);
+        if (null == sessionId) {
+            LOG.info("No session ID associated with OIDC session \"{}\" from backchannel logout found.", sid);
+            return;
+        }
+        LOG.trace("Retrieved mapped session \"{}\" for parsed OID session \"{}\"", sessionId, sid);
+        String sessionFilterString = new StringBuilder().append('(').append(SessionFilter.SESSION_ID).append('=').append(sessionId).append(')').toString();
+        Collection<String> removedSessions;
+        try {
+            removedSessions = services.getServiceSafe(SessiondService.class).removeSessionsGlobally(SessionFilter.create(sessionFilterString));
+        } catch (OXException e) {
+            LOG.error("Unexpected error removing sessions via filter \"{}\" upon bachchannel logout.", sessionFilterString, e);
+            throw e;
+        }
+        if (null == removedSessions || removedSessions.isEmpty()) {
+            LOG.info("No additional sessions with ID \"{}\" (OIDC session \"{}\") removed from nodes.", sessionId, sid);
+        } else {
+            LOG.info("Successfully removed {} session(s) with ID \"{}\" (OIDC session \"{}\") from all nodes via backchannel logout.", I(removedSessions.size()), sessionId, sid);
+        }
+    }
+
+    @Override
+    public void frontchannelLogout(HttpServletRequest request, HttpServletResponse response) throws OXException {
+        LOG.trace("frontchannelLogout(HttpServletRequest request: {}, HttpServletResponse response)", request.getRequestURI());
+        if (false == backend.getBackendConfig().isFrontchannelLogoutEnabled()) {
+            throw OIDCExceptionCode.INVALID_LOGOUT_REQUEST.create("Frontchannel logout disabled by configuration");
+        }
+        /*
+         * validate request issuer
+         */
+        if (false == validateThirdPartyRequest(request)) {
+            LOG.warn("Issuer \"{}\" of front channel logout request could not be validated.", request.getParameter("iss"));
+            throw OIDCExceptionCode.INVALID_LOGOUT_REQUEST.create("Issuer of frontchannel logout request could not be validated");
+        }
+        /*
+         * parse oidc session from request
+         */
+        String sid = request.getParameter("sid");
+        if (Strings.isEmpty(sid)) {
+            LOG.warn("No sid in request {}, unable to process frontchannel logout", request.getRequestURI());
+            throw OIDCExceptionCode.INVALID_LOGOUT_REQUEST.create("No sid in frontchannel logout request");
+        }
+        LOG.trace("Successfully got sid \"{}\" from logout request", sid);
+        /*
+         * lookup matching ox session & logout
+         */
+        String sessionId = stateManagement.getAndRemoveOXSessionId(sid);
+        if (null == sessionId) {
+            LOG.info("No session ID associated with OIDC session \"{}\" from frontchannel logout found.", sid);
+            return;
+        }
+        LOG.trace("Found mapped session \"{}\" for parsed OID session \"{}\"", sessionId, sid);
+        Session session = LoginPerformer.getInstance().lookupSession(sessionId);
+        if (null == session) {
+            LOG.info("Session \"{}\" (OIDC session \"{}\") not found.", sessionId, sid);
+            return;
+        }
+        backend.logoutCurrentUser(session, request, response);
+        LOG.info("Successfully logged out session \"{}\" (OIDC session \"{}\") via frontchannel logout.", sessionId, sid);
+    }
+
     private LogoutRequestInfo loadLogoutRequestInfo(HttpServletRequest request) throws OXException {
         LOG.trace("loadLogoutRequestInfo(HttpServletRequest request: {})", request.getRequestURI());
         // state
@@ -547,4 +662,5 @@ public class OIDCWebSSOProviderImpl implements OIDCWebSSOProvider {
         }
         return logoutRequestInfo;
     }
+
 }
