@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -28,6 +28,7 @@ import com.openexchange.config.ConfigurationService;
 import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.context.ContextService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
+import com.openexchange.groupware.upgrade.SegmentedUpdateService;
 import com.openexchange.hazelcast.serialization.CustomPortableFactory;
 import com.openexchange.lock.LockService;
 import com.openexchange.login.listener.LoginListener;
@@ -44,6 +45,8 @@ import com.openexchange.oidc.impl.OIDCSessionInspectorService;
 import com.openexchange.oidc.impl.OIDCSessionParameterNamesProvider;
 import com.openexchange.oidc.impl.OIDCSessionSsoProvider;
 import com.openexchange.oidc.spi.OIDCCoreBackend;
+import com.openexchange.oidc.state.StateManagement;
+import com.openexchange.oidc.state.impl.CoreStateManagement;
 import com.openexchange.osgi.HousekeepingActivator;
 import com.openexchange.rest.client.httpclient.HttpClientService;
 import com.openexchange.rest.client.httpclient.SpecificHttpClientConfigProvider;
@@ -83,7 +86,8 @@ public class OIDCActivator extends HousekeepingActivator{
             UserService.class,
             SessiondService.class,
             ServerConfigService.class,
-            SessionOAuthTokenService.class
+            SessionOAuthTokenService.class,
+            SegmentedUpdateService.class
         };
     }
 
@@ -92,9 +96,9 @@ public class OIDCActivator extends HousekeepingActivator{
         return true;
     }
 
-    private void getOIDCBackends(ServiceLookup services) {
+    private void getOIDCBackends(ServiceLookup services, StateManagement stateManagement) {
         if (this.oidcBackendRegistry == null) {
-            this.oidcBackendRegistry = new OIDCBackendRegistry(context, services);
+            this.oidcBackendRegistry = new OIDCBackendRegistry(context, services, stateManagement);
             this.oidcBackendRegistry.open();
         }
     }
@@ -117,9 +121,11 @@ public class OIDCActivator extends HousekeepingActivator{
         Logger logger = org.slf4j.LoggerFactory.getLogger(OIDCActivator.class);
         if (config.isEnabled()) {
             logger.info("Starting core OpenID Connect support... ");
-            getOIDCBackends(this);
-            registerService(SessionInspectorService.class, new OIDCSessionInspectorService(oidcBackendRegistry, getService(SessionOAuthTokenService.class)));
-            registerService(AuthenticationFailedHandler.class, new OIDCAuthenticationFailedHandler(), 100);
+            StateManagement stateManagement = new CoreStateManagement(getService(HazelcastInstance.class));
+            getOIDCBackends(this, stateManagement);
+            SessionOAuthTokenService sessionOAuthTokenService = getServiceSafe(SessionOAuthTokenService.class);
+            registerService(SessionInspectorService.class, new OIDCSessionInspectorService(oidcBackendRegistry, sessionOAuthTokenService, stateManagement));
+            registerService(AuthenticationFailedHandler.class, new OIDCAuthenticationFailedHandler(oidcBackendRegistry, sessionOAuthTokenService, stateManagement, this), 100);
             registerService(CustomPortableFactory.class, new PortableAuthenticationRequestFactory(), null);
             registerService(CustomPortableFactory.class, new PortableLogoutRequestFactory(), null);
 
