@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -37,7 +37,6 @@ import org.osgi.service.http.NamespaceException;
 import org.osgi.util.tracker.ServiceTracker;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import com.hazelcast.core.HazelcastInstance;
 import com.openexchange.ajax.AJAXServlet;
 import com.openexchange.ajax.LoginServlet;
 import com.openexchange.ajax.login.LoginConfiguration;
@@ -51,12 +50,14 @@ import com.openexchange.oidc.OIDCExceptionCode;
 import com.openexchange.oidc.OIDCExceptionHandler;
 import com.openexchange.oidc.OIDCWebSSOProvider;
 import com.openexchange.oidc.http.AuthenticationService;
+import com.openexchange.oidc.http.BackchannelLogoutServlet;
+import com.openexchange.oidc.http.FrontchannelLogoutServlet;
 import com.openexchange.oidc.http.InitService;
 import com.openexchange.oidc.http.LogoutService;
 import com.openexchange.oidc.impl.OIDCLoginRequestHandler;
 import com.openexchange.oidc.impl.OIDCLogoutRequestHandler;
 import com.openexchange.oidc.impl.OIDCWebSSOProviderImpl;
-import com.openexchange.oidc.state.impl.CoreStateManagement;
+import com.openexchange.oidc.state.StateManagement;
 import com.openexchange.oidc.tools.OIDCTools;
 import com.openexchange.osgi.service.http.HttpServices;
 import com.openexchange.server.ServiceLookup;
@@ -85,16 +86,16 @@ public class OIDCBackendRegistry extends ServiceTracker<OIDCBackend, OIDCBackend
     private final ConcurrentHashMap<OIDCBackend, Stack<String>> backendServlets;
     private final ConcurrentHashMap<OIDCBackend, Stack<ServiceRegistration<?>>> backendServiceRegistrations;
     private final LoginConfiguration loginConfiguration;
-    private final CoreStateManagement stateManagement;
+    private final StateManagement stateManagement;
 
-    public OIDCBackendRegistry(BundleContext context, ServiceLookup services) {
+    public OIDCBackendRegistry(BundleContext context, ServiceLookup services, StateManagement stateManagement) {
         super(context, OIDCBackend.class, null);
         this.services = services;
         this.backends = new ConcurrentList<>();
         this.backendServlets = new ConcurrentHashMap<>();
         this.backendServiceRegistrations = new ConcurrentHashMap<>();
         this.loginConfiguration = LoginServlet.getLoginConfiguration();
-        this.stateManagement = new CoreStateManagement(services.getService(HazelcastInstance.class));
+        this.stateManagement = stateManagement;
     }
 
     @Override
@@ -134,8 +135,14 @@ public class OIDCBackendRegistry extends ServiceTracker<OIDCBackend, OIDCBackend
             this.registerServlet(servlets, httpService, prefix, new InitService(ssoProvider, exceptionHandler), "init");
             this.registerServlet(servlets, httpService, prefix, new AuthenticationService(ssoProvider, exceptionHandler), "auth");
             this.registerServlet(servlets, httpService, prefix, new LogoutService(ssoProvider, exceptionHandler), "logout");
-            this.registerRequestHandler(oidcBackend, serviceRegistrations, OIDCTools.OIDC_LOGIN, new OIDCLoginRequestHandler(oidcBackend));
-            this.registerRequestHandler(oidcBackend, serviceRegistrations, OIDCTools.OIDC_LOGOUT, new OIDCLogoutRequestHandler(oidcBackend));
+            if (oidcBackend.getBackendConfig().isBackchannelLogoutEnabled()) {
+                this.registerServlet(servlets, httpService, prefix, new BackchannelLogoutServlet(ssoProvider, exceptionHandler), "backchannel_logout");
+            }
+            if (oidcBackend.getBackendConfig().isFrontchannelLogoutEnabled()) {
+                this.registerServlet(servlets, httpService, prefix, new FrontchannelLogoutServlet(ssoProvider, exceptionHandler), "frontchannel_logout");
+            }
+            this.registerRequestHandler(oidcBackend, serviceRegistrations, OIDCTools.OIDC_LOGIN, new OIDCLoginRequestHandler(oidcBackend, stateManagement));
+            this.registerRequestHandler(oidcBackend, serviceRegistrations, OIDCTools.OIDC_LOGOUT, new OIDCLogoutRequestHandler(oidcBackend, stateManagement));
 
             if (!servlets.isEmpty()) {
                 this.backendServlets.putIfAbsent(oidcBackend, servlets);
