@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.oidc.spi;
 
+import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDENTIALS;
 import static com.openexchange.java.Autoboxing.I;
 import java.io.IOException;
 import java.net.MalformedURLException;
@@ -64,17 +65,20 @@ import com.nimbusds.openid.connect.sdk.LogoutRequest;
 import com.nimbusds.openid.connect.sdk.Nonce;
 import com.nimbusds.openid.connect.sdk.OIDCTokenResponse;
 import com.nimbusds.openid.connect.sdk.claims.IDTokenClaimsSet;
+import com.nimbusds.openid.connect.sdk.claims.LogoutTokenClaimsSet;
 import com.nimbusds.openid.connect.sdk.validators.IDTokenValidator;
+import com.nimbusds.openid.connect.sdk.validators.LogoutTokenValidator;
 import com.openexchange.ajax.LoginServlet;
 import com.openexchange.ajax.SessionUtility;
 import com.openexchange.ajax.login.LoginConfiguration;
 import com.openexchange.ajax.login.LoginTools;
 import com.openexchange.authentication.Authenticated;
-import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginInfo;
 import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
+import com.openexchange.groupware.contexts.Context;
+import com.openexchange.groupware.upgrade.SegmentedUpdateService;
 import com.openexchange.java.Strings;
 import com.openexchange.login.LoginRequest;
 import com.openexchange.login.internal.LoginPerformer;
@@ -100,6 +104,7 @@ import com.openexchange.session.oauth.SessionOAuthTokenService;
 import com.openexchange.session.oauth.TokenRefreshConfig;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.sessionstorage.SessionStorageService;
+import com.openexchange.user.User;
 import com.openexchange.user.UserService;
 
 /**
@@ -144,7 +149,7 @@ public abstract class AbstractOIDCBackend implements OIDCBackend {
 
     @Override
     public OIDCExceptionHandler getExceptionHandler() {
-        return new OIDCCoreExceptionHandler(this.getBackendConfig());
+        return new OIDCCoreExceptionHandler(this.getBackendConfig(), Services.getOptionalService(SegmentedUpdateService.class));
     }
 
     @Override
@@ -200,7 +205,14 @@ public abstract class AbstractOIDCBackend implements OIDCBackend {
                 }
                 return httpClientService.getHttpClient(OIDCHttpClientConfig.getClientIdOidc());
             });
-            IDTokenValidator idTokenValidator = new IDTokenValidator(new Issuer(this.getBackendConfig().getOpIssuer()), new ClientID(this.getBackendConfig().getClientID()), expectedJWSAlg, new URL(this.getBackendConfig().getOpJwkSetEndpoint()),  resourceRetriever);
+            OIDCBackendConfig config = getBackendConfig();
+            // @formatter:off
+            IDTokenValidator idTokenValidator = new IDTokenValidator(new Issuer(config.getOpIssuer()),
+                                                                     new ClientID(config.getClientID()),
+                                                                     expectedJWSAlg,
+                                                                     new URL(config.getOpJwkSetEndpoint()),
+                                                                     resourceRetriever);
+            // @formatter:on
             return idTokenValidator.validate(idToken, Strings.isEmpty(nonce) ? null : new Nonce(nonce));
         } catch (BadJOSEException e) {
             throw OIDCExceptionCode.IDTOKEN_VALIDATON_FAILED_CONTENT.create(e, e.getMessage());
@@ -211,6 +223,29 @@ public abstract class AbstractOIDCBackend implements OIDCBackend {
         }
     }
 
+    @Override
+    public LogoutTokenClaimsSet validateLogoutToken(JWT logoutToken) throws OXException {
+        LOG.trace("IDTokenClaimsSet validateLogoutToken(JWT logoutToken: {})", logoutToken.getParsedString());
+        JWSAlgorithm expectedJWSAlg = this.getJWSAlgorithm();
+        try {
+            ResourceRetriever resourceRetriever = new HttpClientResourceRetriever(() -> {
+                HttpClientService httpClientService = Services.getOptionalService(HttpClientService.class);
+                if (httpClientService == null) {
+                    throw new IllegalStateException("Missing service " + HttpClientService.class.getName());
+                }
+                return httpClientService.getHttpClient(OIDCHttpClientConfig.getClientIdOidc());
+            });
+            LogoutTokenValidator logoutTokenValidator = new LogoutTokenValidator(new Issuer(this.getBackendConfig().getOpIssuer()), new ClientID(this.getBackendConfig().getClientID()), expectedJWSAlg, new URL(this.getBackendConfig().getOpJwkSetEndpoint()), resourceRetriever);
+            return logoutTokenValidator.validate(logoutToken);
+        } catch (BadJOSEException e) {
+            throw OIDCExceptionCode.LOGOUTTOKEN_VALIDATON_FAILED.create(e, e.getMessage());
+        } catch (JOSEException e) {
+            throw OIDCExceptionCode.LOGOUTTOKEN_VALIDATON_FAILED.create(e, "");
+        } catch (MalformedURLException e) {
+            throw OIDCExceptionCode.LOGOUTTOKEN_VALIDATON_FAILED.create(e, "Unable to parse JWKSet URL");
+        }
+    }
+
     @Override
     public Scope getScope() {
         String scope = this.getBackendConfig().getScope();
@@ -225,12 +260,64 @@ public abstract class AbstractOIDCBackend implements OIDCBackend {
 
     @Override
     public LoginRequest getLoginRequest(HttpServletRequest request, int userID, int contextID, LoginConfiguration loginConfiguration) throws OXException {
-        String login = userID + "@" + contextID;
+        Optional<String> optLogin = buildLoginFromRequestAttributes(request);
+        String login = optLogin.isPresent() ? optLogin.get() : buildLoginFromLoginInfo(userID, contextID);
         LOG.trace("getLoginRequest(...) login: {}", login);
         String defaultClient = loginConfiguration.getDefaultClient();
         return LoginTools.parseLogin(request, login, null, false, defaultClient, loginConfiguration.isCookieForceHTTPS(), false);
     }
 
+    /**
+     * Tries to build login from given request's attributes; provided that formerly set in 'c.o.oidc.impl.OIDCLoginRequestHandler.OIDCHandler.loginUser()'.
+     *
+     * @param request The HTTP request
+     * @return The login or empty
+     */
+    private Optional<String> buildLoginFromRequestAttributes(HttpServletRequest request) {
+        String contextInfo = (String) request.getAttribute(OIDCTools.CONTEXT_LOGIN_INFO);
+        if (Strings.isEmpty(contextInfo)) {
+            return Optional.empty();
+        }
+
+        String userName = (String) request.getAttribute(OIDCTools.USER_LOGIN_INFO);
+        if (Strings.isEmpty(userName)) {
+            return Optional.empty();
+        }
+
+        return Optional.of(new StringBuilder(userName).append('@').append(contextInfo).toString());
+    }
+
+    /**
+     * Builds the login from stored login information of given user/context.
+     *
+     * @param userID The user identifier
+     * @param contextID The context identifier
+     * @return The login build from login information
+     * @throws OXException If user/context information cannot be obtained
+     */
+    private String buildLoginFromLoginInfo(int userID, int contextID) throws OXException {
+        // Acquire needed services
+        ContextService contextService = Services.getServiceSafe(ContextService.class);
+        UserService userService = Services.getServiceSafe(UserService.class);
+
+        // Get user/context
+        Context context = contextService.getContext(contextID);
+        User user = userService.getUser(userID, context);
+
+        // Grab login info
+        String ctxLoginInfo = null;
+        String[] ctxLoginInfos = context.getLoginInfo();
+        for (int j = 0; ctxLoginInfo == null && j < ctxLoginInfos.length; j++) {
+            String li = ctxLoginInfos[j];
+            if (Strings.parsePositiveInt(li) >= 0) {
+                // Context identifier. Skip...
+            } else {
+                ctxLoginInfo = li;
+            }
+        }
+        return user.getLoginInfo() + '@' + (ctxLoginInfo == null ? Integer.toString(contextID) : ctxLoginInfo);
+    }
+
     @Override
     public Authenticated enhanceAuthenticated(Authenticated defaultAuthenticated, Map<String, String> state) {
         return defaultAuthenticated;
@@ -292,17 +379,20 @@ public abstract class AbstractOIDCBackend implements OIDCBackend {
             if (Strings.isEmpty(contextClaim)) {
                 throw OIDCExceptionCode.UNABLE_TO_LOAD_USERINFO.create("Unable to get a valid context claim: " + config.getContextLookupClaim());
             }
+            LOG.debug("Extracted the following context claim for \"{}\": {}", config.getContextLookupClaim(), contextClaim);
 
             userClaim = jwtClaimsSet.getStringClaim(config.getUserLookupClaim());
             if (Strings.isEmpty(userClaim)) {
                 throw OIDCExceptionCode.UNABLE_TO_LOAD_USERINFO.create("Unable to get a valid user claim: " + config.getUserLookupClaim());
             }
+            LOG.debug("Extracted the following user claim for \"{}\": {}", config.getUserLookupClaim(), userClaim);
         } catch (java.text.ParseException e) {
             throw OIDCExceptionCode.UNABLE_TO_LOAD_USERINFO.create(e, "Failed to parse claim from idToken.");
         }
 
         String contextName = config.getContextLookupNamePart().getFrom(contextClaim, Authenticated.DEFAULT_CONTEXT_INFO);
         String userName = config.getUserLookupNamePart().getFrom(userClaim, userClaim);
+        LOG.debug("Going to use the following names for context/user look-up: context-name={} | user-name={}", contextName, userName);
 
         AuthenticationInfo resultInfo = this.loadUserFromServer(contextName, userName);
         resultInfo.setProperty(AUTH_RESPONSE, tokenResponse.toJSONObject().toJSONString());
@@ -316,7 +406,7 @@ public abstract class AbstractOIDCBackend implements OIDCBackend {
         int contextId = contextService.getContextId(contextInfo);
         if (contextId < 0) {
             LOG.debug("Unknown context for login mapping '{}'", contextInfo);
-            throw LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_CONTEXT_MAPPING.create(contextInfo);
+            throw INVALID_CREDENTIALS.create();
         }
 
         int userId = userService.getUserId(userName, contextService.getContext(contextId));
