@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -146,7 +146,7 @@ public abstract class AbstractOIDCBackendConfig implements OIDCBackendConfig {
 
     @Override
     public List<String> getHosts() {
-        String[] hosts = this.loadStringProperty(OIDCBackendProperty.hosts).split(",");
+        String[] hosts = Strings.splitByComma(this.loadStringProperty(OIDCBackendProperty.hosts));
         return Arrays.asList(hosts);
     }
 
@@ -210,6 +210,16 @@ public abstract class AbstractOIDCBackendConfig implements OIDCBackendConfig {
         return this.loadBooleanProperty(OIDCBackendProperty.tryRecoverStoredTokens);
     }
 
+    @Override
+    public boolean isBackchannelLogoutEnabled() {
+        return this.loadBooleanProperty(OIDCBackendProperty.backchannelLogoutEnabled);
+    }
+
+    @Override
+    public boolean isFrontchannelLogoutEnabled() {
+        return this.loadBooleanProperty(OIDCBackendProperty.frontchannelLogoutEnabled);
+    }
+
     /**
      * Load the given {@link String} property. If a custom backendName is stored, this name
      * is added to the prefix, to load the correct property.
