@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.oidc.spi;
 
+import com.openexchange.groupware.upgrade.SegmentedUpdateService;
 import com.openexchange.oidc.OIDCBackendConfig;
 import com.openexchange.oidc.OIDCExceptionHandler;
 
@@ -32,8 +33,8 @@ import com.openexchange.oidc.OIDCExceptionHandler;
  */
 public class OIDCCoreExceptionHandler extends AbstractOIDCExceptionHandler {
 
-    public OIDCCoreExceptionHandler(OIDCBackendConfig config) {
-        super(config);
+    public OIDCCoreExceptionHandler(OIDCBackendConfig config, SegmentedUpdateService segmentedUpdateService) {
+        super(config, segmentedUpdateService);
     }
     
 }
