@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -23,6 +23,8 @@ package com.openexchange.oidc.tools;
 
 import static com.openexchange.ajax.AJAXServlet.PARAMETER_SESSION;
 import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.java.Autoboxing.L;
+import static com.openexchange.java.Autoboxing.l;
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.net.URI;
@@ -38,6 +40,11 @@ import javax.servlet.http.HttpServletResponse;
 import org.apache.http.client.utils.URIBuilder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import com.nimbusds.jwt.JWTClaimsSet;
+import com.nimbusds.jwt.JWTParser;
+import com.nimbusds.oauth2.sdk.AuthorizationGrant;
+import com.nimbusds.oauth2.sdk.TokenRequest;
+import com.nimbusds.oauth2.sdk.auth.ClientAuthentication;
 import com.nimbusds.oauth2.sdk.token.AccessToken;
 import com.nimbusds.oauth2.sdk.token.RefreshToken;
 import com.nimbusds.oauth2.sdk.token.Tokens;
@@ -59,8 +66,10 @@ import com.openexchange.oidc.OIDCBackend;
 import com.openexchange.oidc.OIDCBackendConfig;
 import com.openexchange.oidc.OIDCExceptionCode;
 import com.openexchange.oidc.osgi.Services;
+import com.openexchange.oidc.state.StateManagement;
 import com.openexchange.session.Session;
 import com.openexchange.session.oauth.OAuthTokens;
+import com.openexchange.session.oauth.SessionOAuthTokenService;
 import com.openexchange.session.oauth.TokenRefreshConfig;
 import com.openexchange.sessiond.ExpirationReason;
 import com.openexchange.sessiond.SessionExceptionCodes;
@@ -573,6 +582,95 @@ public class OIDCTools {
         return new Date(System.currentTimeMillis() + (expiresIn * 1000));
     }
 
+    /**
+     * Remembers an OIDC to OX session id mapping within the given state management instance.
+     *
+     * @param stateManagement The state management instance to use
+     * @param session The session to remember the id association for
+     * @param idToken The ID token to extract the OIDC session id from
+     * @return <code>true</code> if the session ids could be extracted and remembered successfully, <code>false</code>, otherwise
+     */
+    public static boolean rememberSessionIds(StateManagement stateManagement, Session session, String idToken) {
+        /*
+         * extract sid from id token
+         */
+        String sid = optOidcSessionId(idToken);
+        if (Strings.isEmpty(sid)) {
+            LOG.warn("Unable to extract OIDC session from \"{}\", unable to remember session id.", idToken);
+            return false;
+        }
+        /*
+         * derive ttl from refresh token if possible
+         */
+        long ttl = -1l;
+        try {
+            // @formatter:off
+            ttl = l(Services.getServiceSafe(SessionOAuthTokenService.class).getFromSession(session)
+                                                                           .map(tokens -> L(tokens.getExpiresInMillis()))
+                                                                           .orElse(L(-1l)));
+            // @formatter:on
+        } catch (OXException e) {
+            LOG.warn("Unexepcted error getting access token to derive session mapping TTL, falling back to defaults.", e);
+        }
+        try {
+            stateManagement.rememberSession(sid, session.getSessionID(), ttl, TimeUnit.MILLISECONDS);
+            LOG.trace("Successfully remembered oidc to ox session mapping for \"{}\" -> \"{}\" with TTL {}ms.", sid, session.getSessionID(), L(ttl));
+            return true;
+        } catch (OXException e) {
+            LOG.warn("Unexepcted error remembering oidc to ox session mapping for \"{}\": {}", session.getSessionID(), e.getMessage(), e);
+        }
+        return false;
+    }
+
+    /**
+     * Removes an OIDC to OX session id mapping within the given state management instance.
+     *
+     * @param stateManagement The state management instance to use
+     * @param idToken The ID token to extract the OIDC session id from
+     * @return <code>true</code> if the session ids could be extracted and removed successfully, <code>false</code>, otherwise
+     */
+    public static boolean forgetSessionIds(StateManagement stateManagement, String idToken) {
+        /*
+         * extract sid from id token
+         */
+        String sid = optOidcSessionId(idToken);
+        if (Strings.isEmpty(sid)) {
+            LOG.warn("Unable to extract OIDC session from \"{}\", unable to remove session id association.", idToken);
+            return false;
+        }
+        String sessionId;
+        try {
+            sessionId = stateManagement.getAndRemoveOXSessionId(sid);
+        } catch (OXException e) {
+            LOG.warn("Unexepcted error removing oidc to ox session mapping for \"{}\": {}", sid, e.getMessage(), e);
+            return false;
+        }
+        if (null != sessionId) {
+            LOG.trace("Successfully removed oidc to ox session mapping for \"{}\".", sid);
+            return true;
+        }
+        LOG.trace("No previous oidc to ox session mapping for \"{}\" found.", sid);
+        return false;
+    }
+
+    /**
+     * Optionally parses and extracts the <code>sid</code> claim, i.e. the OPs session identifier, from the supplied id token string.
+     *
+     * @param idToken The id token to extract the OIDC session identifier from
+     * @return The OIDC session identifier, or <code>null</code> if none could be extracted
+     */
+    public static String optOidcSessionId(String idToken) {
+        if (Strings.isNotEmpty(idToken)) {
+            try {
+                JWTClaimsSet claimsSet = JWTParser.parse(idToken).getJWTClaimsSet();
+                return null != claimsSet ? claimsSet.getStringClaim("sid") : null;
+            } catch (Exception e) {
+                LOG.debug("Error getting \"sid\" from id token {}", idToken, e);
+            }
+        }
+        return null;
+    }
+
     /**
      * Sets path, query and fragment at the passed {@link URIBuilder} instance to create
      * an URI that can be used to bootstrap a frontend session with optional deep link.
@@ -610,4 +708,63 @@ public class OIDCTools {
         return uriFragment;
     }
 
+    // -------------------------------------------------------------------------------------------------------------------------------------
+
+    /**
+     * Create a <code>toString()</code> representation for given token request.
+     *
+     * @param tokenReq The token request
+     * @return The object providing proper <code>toString()</code> method
+     */
+    public static Object tokeRequestInfo(TokenRequest tokenReq) {
+        if (tokenReq == null) {
+            return "null";
+        }
+
+        return new TokenRequestToString(tokenReq);
+    }
+
+    private static final class TokenRequestToString {
+
+        private final TokenRequest tokenReq;
+
+        TokenRequestToString(TokenRequest tokenReq) {
+            this.tokenReq = tokenReq;
+        }
+
+        @Override
+        public String toString() {
+            return new StringBuilder(512).append('{')
+                .append("uri=").append(tokenReq.getEndpointURI())
+                .append(", clientAuth=").append(toString(tokenReq.getClientAuthentication()))
+                .append(", authzGrant=").append(toString(tokenReq.getAuthorizationGrant()))
+                .append(", scope=").append(tokenReq.getScope())
+                .append(", customParams=").append(tokenReq.getCustomParameters())
+                .append('}').toString();
+        }
+
+        private static String toString(ClientAuthentication clientAuthentication) {
+            if (clientAuthentication == null) {
+                return "null";
+            }
+
+            return new StringBuilder(128).append('{')
+                .append("method=").append(clientAuthentication.getMethod())
+                .append(", clientID=").append(clientAuthentication.getClientID())
+                .append('}').toString();
+        }
+
+        private static String toString(AuthorizationGrant authorizationGrant) {
+            if (authorizationGrant == null) {
+                return "null";
+            }
+
+            return new StringBuilder(64).append('{')
+                .append("type=").append(authorizationGrant.getType())
+                .append('}').toString();
+        }
+    }
+
+    // -------------------------------------------------------------------------------------------------------------------------------------
+
 }
