@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -318,4 +318,24 @@ public interface OIDCBackendConfig {
      */
     boolean tryRecoverStoredTokens();
 
+    /**
+     * Gets a value indicating whether backchannel logout is enabled or not. If enabled, a separate servlet is
+     * registered under <code>/<dispatcherPrefix>/oidc/<backendPath>/backchannel_logout</code>, which will accept
+     * backchannel logout requests from the OP (the "sid" claim is required in the logout token to associate the
+     * OP session with the corresponding OX session properly).
+     * 
+     * @return <code>true</code> if backchannel logout is enabled, <code>false</code>, otherwise
+     */
+    boolean isBackchannelLogoutEnabled();
+
+    /**
+     * Gets a value indicating whether frontchannel logout is enabled or not. If enabled, a separate servlet is
+     * registered under <code>/<dispatcherPrefix>/oidc/<backendPath>/frontchannel_logout</code>, which will accept
+     * frontchannel logout requests from the OP (the "sid" parameter is required in the request to associate the
+     * OP session with the corresponding OX session properly).
+     * 
+     * @return <code>true</code> if frontchannel logout is enabled, <code>false</code>, otherwise
+     */
+    boolean isFrontchannelLogoutEnabled();
+
 }
