@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,7 +21,6 @@
 package com.openexchange.oidc;
 
 import static com.openexchange.java.Autoboxing.I;
-
 import com.nimbusds.oauth2.sdk.token.AccessToken;
 import com.openexchange.authentication.NamePart;
 import com.openexchange.config.lean.Property;
@@ -172,7 +171,22 @@ public enum OIDCBackendProperty implements Property {
      * This is only tried as a fall-back, after token refresh failed with an
      * {@code invalid_grant} error.
      */
-    tryRecoverStoredTokens(OIDCProperty.PREFIX, Boolean.FALSE)
+    tryRecoverStoredTokens(OIDCProperty.PREFIX, Boolean.FALSE),
+    /**
+     * backchannelLogoutEnabled - Configures whether backchannel logout is enabled or not. If enabled, a
+     * separate servlet is registered under <code>/<dispatcherPrefix>/oidc/<backendPath>/backchannel_logout</code>,
+     * which will accept backchannel logout requests from the OP (the "sid" claim is required in the logout token
+     * to associate the OP session with the corresponding OX session properly).
+     */
+    backchannelLogoutEnabled(OIDCProperty.PREFIX, Boolean.FALSE),
+    /**
+     * frontchannelLogoutEnabled - Configures whether frontchannel logout is enabled or not. If enabled, a
+     * separate servlet is registered under <code>/<dispatcherPrefix>/oidc/<backendPath>/frontchannel_logout</code>,
+     * which will accept frontchannel logout requests from the OP (the "sid" parameter is required in the request
+     * to associate the OP session with the corresponding OX session properly).
+     */
+    frontchannelLogoutEnabled(OIDCProperty.PREFIX, Boolean.FALSE),
+
     ;
 
     private final String fqn;
