@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -68,4 +68,22 @@ public interface StateManagement {
      */
     LogoutRequestInfo getAndRemoveLogoutRequestInfo(String state) throws OXException;
 
+    /**
+     * Removes the mapping for the supplied OIDC session id.
+     * 
+     * @param oidcSid The OIDC session id to remove
+     * @return The OX session id previously associated with the OIDC session id, or <code>null</code> if no such mapping existed
+     */
+    String getAndRemoveOXSessionId(String oidcSid) throws OXException;
+
+    /**
+     * Remembers an OIDC to OX session id mapping.
+     * 
+     * @param oidcSid The OIDC session id to remember
+     * @param oxSessionId The OX session id to associate with the OIDC session id
+     * @param ttl The TTL to use for the mapping, or <code>-1</code> to use the map defaults
+     * @param timeUnit The timeunit of the given TTL parameter
+     */
+    void rememberSession(String oidcSid, String oxSessionId, long ttl, TimeUnit timeUnit) throws OXException;
+
 }
